@@ -77,16 +77,17 @@ static void show_help(const char* argv0)
                "These suffices are recognized: jpg, jpeg, png, y4m."
                "\n"
                "Options:\n"
-               "  -h, --help              show help\n"
-               "  -q, --quality           quality (for JPEG output)\n"
-               "  -d, --decoder ID        use a specific decoder (see --list-decoders)\n"
-               "      --with-aux          also write auxiliary images (e.g. depth images)\n"
-               "      --with-xmp          write XMP metadata to file (output filename with .xmp suffix)\n"
-               "      --with-exif         write EXIF metadata to file (output filename with .exif suffix)\n"
-               "      --skip-exif-offset  skip EXIF metadata offset bytes\n"
-               "      --no-colons         replace ':' characters in auxiliary image filenames with '_'\n"
-               "      --list-decoders     list all available decoders (built-in and plugins)\n"
-               "      --quiet             do not output status messages to console\n";
+               "  -h, --help                     show help\n"
+               "  -q, --quality                  quality (for JPEG output)\n"
+               "  -d, --decoder ID               use a specific decoder (see --list-decoders)\n"
+               "      --with-aux                 also write auxiliary images (e.g. depth images)\n"
+               "      --with-xmp                 write XMP metadata to file (output filename with .xmp suffix)\n"
+               "      --with-exif                write EXIF metadata to file (output filename with .exif suffix)\n"
+               "      --skip-exif-offset         skip EXIF metadata offset bytes\n"
+               "      --no-colons                replace ':' characters in auxiliary image filenames with '_'\n"
+               "      --list-decoders            list all available decoders (built-in and plugins)\n"
+               "      --quiet                    do not output status messages to console\n"
+               "  -C, --chroma-upsampling ALGO   Force chroma upsampling algorithm (nn = nearest-neighbor / bilinear)\n";
 }
 
 
@@ -114,6 +115,8 @@ int option_with_exif = 0;
 int option_skip_exif_offset = 0;
 int option_list_decoders = 0;
 
+std::string chroma_upsampling;
+
 static struct option long_options[] = {
     {(char* const) "quality",          required_argument, 0,                        'q'},
     {(char* const) "strict",           no_argument,       0,                        's'},
@@ -125,7 +128,8 @@ static struct option long_options[] = {
     {(char* const) "skip-exif-offset", no_argument,       &option_skip_exif_offset, 1},
     {(char* const) "no-colons",        no_argument,       &option_no_colons,        1},
     {(char* const) "list-decoders",    no_argument,       &option_list_decoders,    1},
-    {(char* const) "help",             no_argument,       0,                        'h'}
+    {(char* const) "help",             no_argument,       0,                        'h'},
+    {(char* const) "chroma-upsampling", required_argument, 0,                     'C'},
 };
 
 
@@ -154,6 +158,12 @@ void list_all_decoders()
 
   std::cout << "AVIF decoders:\n";
   list_decoders(heif_compression_AV1);
+
+#if WITH_UNCOMPRESSED_CODEC
+  std::cout << "uncompressed: yes\n";
+#else
+  std::cout << "uncompressed: no\n";
+#endif
 }
 
 
@@ -177,7 +187,7 @@ int main(int argc, char** argv)
   //while ((opt = getopt(argc, argv, "q:s")) != -1) {
   while (true) {
     int option_index = 0;
-    int c = getopt_long(argc, argv, "hq:sd:", long_options, &option_index);
+    int c = getopt_long(argc, argv, "hq:sd:C:", long_options, &option_index);
     if (c == -1) {
       break;
     }
@@ -198,6 +208,18 @@ int main(int argc, char** argv)
       case 'h':
         show_help(argv[0]);
         return 0;
+      case 'C':
+        chroma_upsampling = optarg;
+        if (chroma_upsampling != "nn" &&
+            chroma_upsampling != "nearest-neighbor" &&
+            chroma_upsampling != "bilinear") {
+          fprintf(stderr, "Undefined chroma upsampling algorithm.\n");
+          exit(5);
+        }
+        if (chroma_upsampling == "nn") { // abbreviation
+          chroma_upsampling = "nearest-neighbor";
+        }
+        break;
     }
   }
 
@@ -360,6 +382,15 @@ int main(int argc, char** argv)
     decode_options->strict_decoding = strict_decoding;
     decode_options->decoder_id = decoder_id;
 
+    if (chroma_upsampling=="nearest-neighbor") {
+      decode_options->color_conversion_options.preferred_chroma_upsampling_algorithm = heif_chroma_upsampling_nearest_neighbor;
+      decode_options->color_conversion_options.only_use_preferred_chroma_algorithm = true;
+    }
+    else if (chroma_upsampling=="bilinear") {
+      decode_options->color_conversion_options.preferred_chroma_upsampling_algorithm = heif_chroma_upsampling_bilinear;
+      decode_options->color_conversion_options.only_use_preferred_chroma_algorithm = true;
+    }
+
     int bit_depth = heif_image_handle_get_luma_bits_per_pixel(handle);
     if (bit_depth < 0) {
       heif_decoding_options_free(decode_options);
@@ -505,7 +536,7 @@ int main(int argc, char** argv)
 
             std::string auxType = std::string(auxTypeC);
 
-            heif_image_handle_free_auxiliary_types(aux_handle, &auxTypeC);
+            heif_image_handle_release_auxiliary_type(aux_handle, &auxTypeC);
 
             std::ostringstream s;
             s << numbered_output_filename_stem;
