@@ -55,8 +55,18 @@ static int usage(const char* command)
 }
 
 
+class LibHeifInitializer {
+public:
+  LibHeifInitializer() { heif_init(nullptr); }
+  ~LibHeifInitializer() { heif_deinit(); }
+};
+
+
 int main(int argc, char** argv)
 {
+  // This takes care of initializing libheif and also deinitializing it at the end to free all resources.
+  LibHeifInitializer initializer;
+
   int opt;
   int size = 512; // default thumbnail size
   bool thumbnail_from_primary_image_only = false;
@@ -158,10 +168,10 @@ int main(int argc, char** argv)
   int input_width = heif_image_handle_get_width(image_handle);
   int input_height = heif_image_handle_get_height(image_handle);
 
-  int thumbnail_width = input_width;
-  int thumbnail_height = input_height;
-
   if (input_width > size || input_height > size) {
+    int thumbnail_width;
+    int thumbnail_height;
+
     if (input_width > input_height) {
       thumbnail_height = input_height * size / input_width;
       thumbnail_width = size;
